/***** alignment.h ************************************************
 * Description: Header file for pairwise optimal sequence 
 *              alignment.
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * File created on Thu Dec 16 20:33:47 2004.
 *****************************************************************/
#include <stdio.h>

#define GAP_OPENING_DNA -5
#define GAP_EXTENSION_DNA -2
#define GAP_OPENING_PROTEIN -11
#define GAP_EXTENSION_PROTEIN -1
#define MATCH 1
#define MISMATCH -3
#define LINE_LENGTH 60
#define VERSION "0.30"
#define GLOBAL 1
#define LOCAL 2
#define OVERLAP 3
#define KERROR 4

/* definition of substitution matrix; the score for
 * residues x and y can be accessed as mat[dic[x]][dic[y]];
 * the character corresponding to the ith row or column in
 * mat is represented as revdic[i]
 */
typedef struct substitutionMatrix{
  float **mat;
  int size;
  int *dic;
  char *revdic;
} SubstitutionMatrix;

typedef struct dpCell{
  long id;                     /* identifier for reconstructing matrix coordinate */
  double s;                    /* score */
  short int next;              /* pointer to next matrix; takes values 0, 1, 2 */
  unsigned int isVisited : 1;  /* keep track of trace back paths */
} DpCell;

typedef struct dpMatrix{
  char *hseq;     /* horizontal sequence */
  char *vseq;     /* vertical sequence */
  DpCell ***mat;
} DpMatrix;

typedef struct matrixPosition{
  int i;
  int j;
  int k;
} MatrixPosition;

typedef struct seqList{
  char c;
  struct seqList *next;
} SeqList;

typedef struct alignment{
  float score;
  int length1;
  int length2;
  int start1;
  int start2;
  int end1;
  int end2;
  int numerr;
  char *name1;
  char *name2;
  SeqList *head1;
  SeqList *head2;
  char *strand;
  struct alignment *next;
} Alignment;

typedef struct matchList{
  int qi;                    /* query index */
  int ti;                    /* text index */
  struct matchList *next;    /* pointer to next element in list */
} MatchList;

/* prototypes alignment.c */
DpMatrix *generateDpMatrix(int m, int n, DpMatrix *dpm);
void freeAlignment(Alignment *al);
void freeSubstitutionMatrix(SubstitutionMatrix *sm);
void freeDpm(DpMatrix *dpm);
int max(int a, int b);
int numLetters(int x);
SubstitutionMatrix *constructDnaScoreMatrix(SubstitutionMatrix *sm, int matchScore, int mismatchScore);
/* int round(float x); */
void decode(int n);
int numMatch(int x, int y);
int min(int a, int b);
SubstitutionMatrix *readSubstitutionMatrix(SubstitutionMatrix *sm, FILE *fp);
void printAlignment(Alignment *al, FILE *fp, SubstitutionMatrix *sm, Args args);
Alignment *traceBack(DpMatrix *dpm, Alignment *al, MatrixPosition *pos, int startQuery, int startSubject, int maxError);
void writeDpm(DpMatrix *dpm);
void setAlignmentMode(int mode);

/* prototypes globalAlignment.c */
Alignment *global(char *seq1, char *seq2, float opening, float extension, SubstitutionMatrix *sm);

/* prototypes localAlignment.c */
int searchStartLocal(DpCell **dpm, int l1, int l2, int *ii, int *jj);

/* prototypes overlapAlignment.c */
Alignment *overlap(char *seq1, char *seq2, float open, float extend, SubstitutionMatrix *sm);

/* prototypes localAlignment.c */
Alignment *local(char *seq1, char *seq2, float open, float extend, SubstitutionMatrix *sm, Args args);

/* prototypes kerrorAlignment.c */
Alignment *alignKerror(char *subject, char *query, int lb, int rb, SubstitutionMatrix *sm, Args args);
